#!/usr/bin/python

import sys
import cStringIO
import gzip

# parses wadokujt: Japanese - German dictionary.
# the file comes as gz-archive. This file holds the records in the
# following format:
#    Japanese /gloss/gloss/.../
# encoded in UTF-8
# This script downloads and converts this file to ding syntax


# parse command-line arguments
sourceType = sys.argv[1]
source = sys.argv[2]
targetFile = sys.argv[3]

if sourceType == 'download':
	import urllib

try:
	if sourceType == 'download':
		print "Opening URL %s..." % source
		f = urllib.urlopen(source)
	else:
		print "Opening local file %s..." % source
		f = open(source, 'r')
	print "  done"
	print "Reading data..."
	data = f.read()
	print "  done"
	f.close()
except:
	print "error while retrieving data from %s:" % source
	print sys.exc_info()[0]
	sys.exit(1)

# data now contains the bytes of the gz-file
try:
	print "Decompressing and decoding data..."
	data = unicode(gzip.GzipFile(None, "r", 0, cStringIO.StringIO(data)).read(), 'utf8')
	print "  done"
except:
	print "error while decompressing and decoding data:"
	print sys.exc_info()[0]
	sys.exit(1)

# data now contains the dictionary-file => parse it
try:

	print "Parsing and writing data to local file %s (utf-8)..." % targetFile
	f = open(targetFile, 'w')
	lines = data.splitlines()
	lines = lines[1:]
	for line in lines:
		if line.startswith("#"):
			continue
			
		components = line.split("/")		
		japanese = components[0].strip()
		glosses = components[1:-1]	# german parts

		german = ""
		firstComment = False
		for gloss in glosses:
			# a new sense
			if gloss.startswith("["):
				if len(german) > 0 and not firstComment:
					line = "%s::%s\n" % (japanese, german)
					f.write(line.encode('utf8'))
					german = ""
					firstComment = False
			
			# 
			else:
				if len(german) == 0 and gloss.startswith("{"):
					firstComment = True
					
				if len(german) > 0:
					if gloss.startswith("(") or gloss.startswith("{") or gloss.endswith(")") or firstComment:
						german = german + " "
					else:
						german = german + "; "
					firstComment = False
				
				if gloss.startswith("(<") or gloss.endswith(">)"):
					gloss = gloss.replace("(<", "{")
					gloss = gloss.replace(">)", "}")
				
				german = german + gloss
				
		if len(german) > 0:
			line = "%s::%s\n" % (japanese, german)
			f.write(line.encode('utf8'))
			german = ""

	f.close()
	print "  done"
except:
	print "error while writing data to %s:" % targetFile
	print sys.exc_info()[0]
	import traceback
	print traceback.extract_tb(sys.exc_info()[2])
	print gloss
	print line.encode('utf8')
	sys.exit(1)


sys.exit(0)
